import json
from abc import ABC
from dataclasses import dataclass
from typing import List


@dataclass(kw_only=True)
class AbstractConfig(ABC):
    name: str
    reshaper: List[str]
    plotter: List[str]
    filename: str
    experiment_class = None  # type: ignore
    experiment_module: str

    def to_json(self) -> str:
        """return a dictionary of instance variables"""
        return json.dumps(vars(self), indent=4)



from typing import Iterator, Tuple
import json
import sys
from tqdm import tqdm
import pathlib
from logging import Logger


def jsonlreader(filename: str, logger: Logger = None) -> Iterator[Tuple[int, dict]]:  # noqa E501
    msg = "You can only use jsonl reader w/ json file"
    assert pathlib.Path(filename).suffix == ".jsonl", msg
    with open(filename, "r") as inf:
        desc_ = "reading {}".format(filename)
        for linenumber, line in tqdm(enumerate(inf), desc=desc_):
            try:
                yield (linenumber, json.loads(line))
            except json.decoder.JSONDecodeError as e:
                msg = "[*] Decode error on line {} of file {}".format(
                    linenumber, filename)
                sys.stderr.write(msg)
                if logger is not None:
                    logger.error(e)


if __name__ == "__main__":

    for linenumber, line in jsonlreader("test/fixtures/data/oren.jsonl"):
        print(line)



class ConfigValidator(object):
    def __init__(self) -> None:
        pass

    @staticmethod
    def is_valid_config_file(config: dict) -> bool:

        required_fields = [
            "name",
            "reshaper",
            "plotter",
            "experiment_module",
            "experiment_class",
        ]

        for field in required_fields:
            assert field in config, "Could not find field {}".format(field)

        assert " " not in config["name"], "Name cant have spaces"
        assert "/" not in config["name"], "Name cant have slashes"

        return True



import json
import os
import time
from typing import List

import pandas as pd  # type: ignore

from naveen.experiment.config.abstract_config import (  # type: ignore # noqa: E501
    AbstractConfig,
)
from naveen.logger import get_logger


class ExperimentWriter(object):
    def __init__(
        self,
        config: AbstractConfig,
        output_directory: str = None,  # type: ignore
    ) -> None:
        self.config = config
        if output_directory is None:
            output_directory = "results/{}".format(int(time.time()))

            self.output_directory = output_directory
        else:
            self.output_directory = output_directory

        if not os.path.exists(output_directory):
            os.makedirs(output_directory)

        logger = get_logger()

        logger.info(
            "[*] Writing experiment to {}".format(self.output_directory)
        )

    def write_results(self, results: List[dict]) -> None:
        df = pd.DataFrame(results)

        outpath = os.path.join(self.output_directory, self.config.name + ".csv")
        df.to_csv(outpath, index=False)
        outconfigpath = os.path.join(self.output_directory, "config.json")
        with open(outconfigpath, "w") as of:
            of.write(self.config.to_json())
